@@ -231,7 +231,14 @@ typedef long int32;
 #error "No 4 byte type, you lose."
 #endif
 #if SIZEOF_LONG == 8
+#ifdef OS_darwin
+#ifndef _UINT64
+typedef uint64_t uint64;
+#define _UINT64
+#endif
+#else
 typedef unsigned long uint64;
+#endif
 typedef long int64;
 #define VAL64(a) (a ## l)
 #define UVAL64(a) (a ## ul)
