@@ -1,8 +1,28 @@
-/* This file is part of STonX, the Atari ST Emulator for Unix/X
- * ============================================================
- * STonX is free software and comes with NO WARRANTY - read the file
- * COPYING for details
+/*
+ * blitter.cpp - Atari Blitter emulation code
  *
+ * Copyright (c) 2001-2004 Petr Stehlik of ARAnyM dev team (see AUTHORS)
+ *
+ * Based on work by Martin Griffiths for the STonX, see below.
+ * 
+ * This file is part of the ARAnyM project which builds a new and powerful
+ * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
+ *
+ * ARAnyM is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ARAnyM is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ARAnyM; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
+/*
  *	Blitter Emulator,
  *	Martin Griffiths, 1995/96.
  *	
@@ -14,20 +34,22 @@
  *	
  */
 
-/*
-#define BLITTER_MEMMOVE		// this replaces complicated logic of word-by-word copying by simple and fast memmove()
-#define BLITTER_SDLBLIT		// this accelerates screen to screen blits with host VGA hardware accelerated routines (using SDL_BlitSurface)
-*/
+
+#undef BLITTER_MEMMOVE		// when defined it replaces complicated logic of word-by-word copying by simple and fast memmove()
+#undef BLITTER_SDLBLIT		// when defined it accelerates screen to screen blits with host VGA hardware accelerated routines (using SDL_BlitSurface)
 
 #include "sysdeps.h"
 #include "hardware.h"
 #include "cpu_emulation.h"
 #include "memory.h"
 #include "blitter.h"
-#include <SDL.h>
+#include "icio.h"
+
 #define DEBUG 0
 #include "debug.h"
 
+#include <SDL.h>
+
 #if DEBUG
 #define SHOWPARAMS												\
 {       fprintf(stderr,"Source Address:%X\n",source_addr);		\
@@ -51,26 +73,40 @@
 }
 #endif
 
-typedef uae_u16	UW;
-typedef uae_u8	UB;
+typedef uint16	UW;
+typedef uint8	UB;
 typedef char	B;
 
 #define ADDR(a)		(a)
 
-BLITTER::BLITTER(void) {
+BLITTER::BLITTER(memptr addr, uint32 size) : BASE_IO(addr, size)
+{
+	reset();
 }
 
-UW BLITTER::LM_UW(uaecptr addr) {
+void BLITTER::reset()
+{
+	// halftone_ram[16];
+	end_mask_1 = end_mask_2 = end_mask_3 = 0;
+	NFSR = FXSR = 0;
+	x_count = y_count = 0;
+	hop = op = line_num = skewreg = 0;
+	halftone_curroffset = halftone_direction = 0;
+	source_x_inc = source_y_inc = dest_x_inc = dest_y_inc = 0;
+	source_addr = 0;
+	dest_addr = 0;
+	blit = false;
+}
+
+UW BLITTER::LM_UW(memptr addr) {
 	return ReadAtariInt16(addr);	//??
 }
 
-void BLITTER::SM_UW(uaecptr addr, UW value) {
-#if DEBUG
-	if (addr <= 0x800 || addr >= 0xe00000) {
-		D(bug("Blitter Error! Tries to write to %06lx", addr));
-		exit(-1);
+void BLITTER::SM_UW(memptr addr, UW value) {
+	if (addr <= 0x800 || (addr >= 0x0e00000 && addr < 0x1000000)) {
+		D(bug("Blitter tried to write to %06lx", addr));
+		return;
 	}
-#endif
 	WriteAtariInt16(addr, value);	//??
 }
 
@@ -272,13 +308,13 @@ HOP_OPS(_HOP_3_OP_15_P,(0xffff) ,source_buffer <<=16,source_buffer |= ((unsigned
 void hop2op3p( BLITTER& b )
 {
 #if BLITTER_MEMMOVE
-	if (videl.getScreenBpp() == 16) {
+	if (getVIDEL ()->getBpp() == 16) {
 #if BLITTER_SDLBLIT
 		if (b.source_addr >= ARANYMVRAMSTART && b.dest_addr >= ARANYMVRAMSTART) {
 			SDL_Rect src, dest;
 			int src_offset = b.source_addr - ARANYMVRAMSTART;
 			int dest_offset = b.dest_addr - ARANYMVRAMSTART;
-			int VidelScreenWidth = videl.getScreenWidth();
+			int VidelScreenWidth = getVIDEL ()->getScreenWidth();
 			src.x = (src_offset % (2*VidelScreenWidth))/2;
 			src.y = (src_offset / (2*VidelScreenWidth));
 			src.w = dest.w = b.x_count;
@@ -295,7 +331,7 @@ void hop2op3p( BLITTER& b )
 #endif /* BLITTER_SDLBLIT */
 		do
 		{
-			memmove(get_real_address_direct(b.dest_addr), get_real_address_direct(b.source_addr), b.x_count*2);
+			memmove(phys_get_real_address(b.dest_addr), phys_get_real_address(b.source_addr), b.x_count*2);
 			b.source_addr += ((b.x_count-1)*b.source_x_inc)+b.source_y_inc;
 			b.dest_addr += ((b.x_count-1)*b.dest_x_inc)+b.dest_y_inc;
 		} while (--b.y_count > 0);
@@ -308,7 +344,7 @@ void hop2op3p( BLITTER& b )
 void hop2op3n( BLITTER& b )
 {
 #if BLITTER_MEMMOVE
-	if (videl.getScreenBpp() == 16) {
+	if (getVIDEL ()->getBpp() == 16) {
 		b.source_addr += ((b.x_count-1)*b.source_x_inc);
 		b.dest_addr += ((b.x_count-1)*b.dest_x_inc);
 #if BLITTER_SDLBLIT
@@ -318,7 +354,7 @@ void hop2op3n( BLITTER& b )
 			SDL_Rect src, dest;
 			int src_offset = b.source_addr - ARANYMVRAMSTART;
 			int dest_offset = b.dest_addr - ARANYMVRAMSTART;
-			int VidelScreenWidth = videl.getScreenWidth();
+			int VidelScreenWidth = getVIDEL ()->getScreenWidth();
 			src.x = (src_offset % (2*VidelScreenWidth))/2;
 			src.y = (src_offset / (2*VidelScreenWidth));
 			src.w = dest.w = b.x_count;
@@ -333,7 +369,7 @@ void hop2op3n( BLITTER& b )
 #endif /* BLITTER_SDLBLIT */
 		do
 		{
-			memmove(get_real_address_direct(b.dest_addr), get_real_address_direct(b.source_addr), b.x_count*2);
+			memmove(phys_get_real_address(b.dest_addr), phys_get_real_address(b.source_addr), b.x_count*2);
 			b.source_addr += ((b.x_count)*b.source_x_inc)+b.source_y_inc;
 			b.dest_addr += ((b.x_count-1)*b.dest_x_inc)+b.dest_y_inc;
 		} while (--b.y_count > 0);
@@ -360,264 +396,133 @@ static void (*do_hop_op_P[4][16])( BLITTER& ) =
 };
 
 
-static const int HW = 0xff8a00;
-
-uae_u8 BLITTER::handleRead(uaecptr addr) {
-	addr -= HW;
-	if (addr < 0 || addr > 0x3d)
-		return 0;
-
-/*
-	if (blit) {
-		Do_Blit();
-		blit = false;
-	}
-*/
-
-	if (addr < 0x20) {
-		uae_u16 hfr = halftone_ram[addr / 2];
-		if (addr % 1)
-			return hfr & 0x00ff;
-		else
-			return hfr >> 8;
-	}
-
-	D(bug("Blitter reads %x at %06x", addr+HW, showPC()));
+uint8 BLITTER::handleRead(memptr addr) {
+	addr -= getHWoffset();
 
 	switch(addr) {
-		case 0x20: return source_x_inc >> 8;
-		case 0x21: return source_x_inc;
-		case 0x22: return source_y_inc >> 8;
-		case 0x23: return source_y_inc;
-		case 0x24: return source_addr >> 24;
-		case 0x25: return source_addr >> 16;
-		case 0x26: return source_addr >> 8;
-		case 0x27: return source_addr;
-		case 0x28: return LOAD_B_ff8a28();
-		case 0x29: return LOAD_B_ff8a29();
-		case 0x2a: return LOAD_B_ff8a2a();
-		case 0x2b: return LOAD_B_ff8a2b();
-		case 0x2c: return LOAD_B_ff8a2c();
-		case 0x2d: return LOAD_B_ff8a2d();
-		case 0x2e: return dest_x_inc >> 8;
-		case 0x2f: return dest_x_inc;
-		case 0x30: return dest_y_inc >> 8;
-		case 0x31: return dest_y_inc;
-		case 0x32: return LOAD_B_ff8a32();
-		case 0x33: return LOAD_B_ff8a33();
-		case 0x34: return LOAD_B_ff8a34();
-		case 0x35: return LOAD_B_ff8a35();
-		case 0x36: return LOAD_B_ff8a36();
-		case 0x37: return LOAD_B_ff8a37();
-		case 0x38: return LOAD_B_ff8a38();
-		case 0x39: return LOAD_B_ff8a39();
-		case 0x3a: return LOAD_B_ff8a3a();
-		case 0x3b: return LOAD_B_ff8a3b();
+		case 0x3a: return hop;
+		case 0x3b: return op;
 		case 0x3c: return LOAD_B_ff8a3c();
-		case 0x3d: return LOAD_B_ff8a3d();
+		case 0x3d: return skewreg;
 	}
 
+	panicbug("Blitter tried to read byte from register %x at %06x", addr+getHWoffset(), showPC());
+
 	return 0;
 }
 
-void BLITTER::handleWrite(uaecptr addr, uae_u8 value) {
-	addr -= HW;
-	if (addr < 0 || addr > 0x3d)
-		return;
+uae_u16 BLITTER::handleReadW(uaecptr addr) {
+	addr -= getHWoffset();
 
 	if (addr < 0x20) {
-		uae_u16 hfr = halftone_ram[addr / 2];
-		if (addr & 1)
-			hfr = (hfr & 0xff00) | value;
-		else
-			hfr = (hfr & 0x00ff) | (value << 8);
-		halftone_ram[addr / 2] = hfr;
-		return;
+		return halftone_ram[addr / 2];
 	}
 
-	D(bug("Blitter writes: %x = %d ($%x) at %06x", addr+HW, value, value, showPC()));
+	switch(addr) {
+		case 0x20: return source_x_inc;
+		case 0x22: return source_y_inc;
+		case 0x28: return end_mask_1;
+		case 0x2a: return end_mask_2;
+		case 0x2c: return end_mask_3;
+		case 0x2e: return dest_x_inc;
+		case 0x30: return dest_y_inc;
+		case 0x36: return x_count;
+		case 0x38: return y_count;
+		case 0x3a: // fallthrough
+		case 0x3c: return BASE_IO::handleReadW(addr+getHWoffset());
+	}
+
+	panicbug("Blitter tried to read word from register %x at %06x", addr+getHWoffset(), showPC());
+
+	return 0;
+}
+
+uae_u32 BLITTER::handleReadL(uaecptr addr) {
+	addr -= getHWoffset();
+
+	if (addr < 0x20) {
+		return BASE_IO::handleReadL(addr+getHWoffset());
+	}
+
+	switch(addr) {
+		case 0x24: return source_addr;
+		case 0x32: return dest_addr;
+	}
+
+	panicbug("Blitter tried to read long word from register %x at %06x", addr+getHWoffset(), showPC());
+
+	return 0;
+}
+
+void BLITTER::handleWrite(memptr addr, uint8 value) {
+	addr -= getHWoffset();
 
 	switch(addr) {
-		case 0x20: source_x_inc = (source_x_inc & 0x00ff) | (value << 8); break;
-		case 0x21: source_x_inc = (source_x_inc & 0xff00) | (value & 0xfe); break;
-		case 0x22: source_y_inc = (source_y_inc & 0x00ff) | (value << 8); break;
-		case 0x23: source_y_inc = (source_y_inc & 0xff00) | (value & 0xfe); break;
-		case 0x24: source_addr = (source_addr & 0x00ffffff) | (value << 24); break;
-		case 0x25: source_addr = (source_addr & 0xff00ffff) | (value << 16); break;
-		case 0x26: source_addr = (source_addr & 0xffff00ff) | (value << 8); break;
-		case 0x27: source_addr = (source_addr & 0xffffff00) | (value & 0xfe); break;	// ignore LSB
-		case 0x28: STORE_B_ff8a28(value); break;
-		case 0x29: STORE_B_ff8a29(value); break;
-		case 0x2a: STORE_B_ff8a2a(value); break;
-		case 0x2b: STORE_B_ff8a2b(value); break;
-		case 0x2c: STORE_B_ff8a2c(value); break;
-		case 0x2d: STORE_B_ff8a2d(value); break;
-		case 0x2e: dest_x_inc = (dest_x_inc & 0x00ff) | (value << 8); break;
-		case 0x2f: dest_x_inc = (dest_x_inc & 0xff00) | (value & 0xfe); break;
-		case 0x30: dest_y_inc = (dest_y_inc & 0x00ff) | (value << 8); break;
-		case 0x31: dest_y_inc = (dest_y_inc & 0xff00) | (value & 0xfe); break;
-		case 0x32: STORE_B_ff8a32(value); break;
-		case 0x33: STORE_B_ff8a33(value); break;
-		case 0x34: STORE_B_ff8a34(value); break;
-		case 0x35: STORE_B_ff8a35(value); break;
-		case 0x36: STORE_B_ff8a36(value); break;
-		case 0x37: STORE_B_ff8a37(value); break;
-		case 0x38: STORE_B_ff8a38(value); break;
-		case 0x39: STORE_B_ff8a39(value); break;
 		case 0x3a: STORE_B_ff8a3a(value); break;
 		case 0x3b: STORE_B_ff8a3b(value); break;
 		case 0x3c: STORE_B_ff8a3c(value); break;
 		case 0x3d: STORE_B_ff8a3d(value); break;
+		default:
+			panicbug("Blitter tried to write byte %d to register %x at %06x", value, addr+getHWoffset(), showPC());
 	}
 
-}
-
-B BLITTER::LOAD_B_ff8a28(void)
-{	return (end_mask_1 >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a29(void)
-{	return end_mask_1 & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a2a(void)
-{	return (end_mask_2 >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a2b(void)
-{	return end_mask_2 & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a2c(void)
-{	return (end_mask_3 >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a2d(void)
-{	return end_mask_3 & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a32(void)
-{	return (dest_addr >> 24);
-}
-
-B BLITTER::LOAD_B_ff8a33(void)
-{	return (dest_addr >> 16) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a34(void)
-{	return (dest_addr >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a35(void)
-{	return (dest_addr) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a36(void)
-{	return (x_count >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a37(void)
-{	return (x_count) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a38(void)
-{	return (y_count >> 8) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a39(void)
-{	return (y_count) & 0xff;
-}
-
-B BLITTER::LOAD_B_ff8a3a(void)
-{	return (B) hop;
-}
-
-B BLITTER::LOAD_B_ff8a3b(void)
-{	return (B) op;
-}
-
-B BLITTER::LOAD_B_ff8a3c(void)
-{
 	if (blit) {
 		Do_Blit();
 		blit = false;
 	}
-	return (B) line_num & 0x3f;
 }
 
-B BLITTER::LOAD_B_ff8a3d(void)
-{	
-	return (B) skewreg;
+void BLITTER::handleWriteW(uaecptr addr, uint16 value) {
+	addr -= getHWoffset();
+
+	if (addr < 0x20) {
+		halftone_ram[addr / 2] = value;
+		return;
+	}
+
+	switch(addr) {
+		case 0x20: source_x_inc = value; break;
+		case 0x22: source_y_inc = value; break;
+		case 0x28: end_mask_1 = value; break;
+		case 0x2a: end_mask_2 = value; break;
+		case 0x2c: end_mask_3 = value; break;
+		case 0x2e: dest_x_inc = value; break;
+		case 0x30: dest_y_inc = value; break;
+		case 0x36: x_count = value; break;
+		case 0x38: y_count = value; break;
+		case 0x3a: STORE_B_ff8a3a(value >> 8); STORE_B_ff8a3b(value); break;
+		case 0x3c: STORE_B_ff8a3c(value >> 8); STORE_B_ff8a3d(value); break;
+		default:
+			panicbug("Blitter tried to write word %d to register %x at %06x", value, addr+getHWoffset(), showPC());
+	}
+
+	if (blit) {
+		Do_Blit();
+		blit = false;
+	}
 }
 
-void BLITTER::STORE_B_ff8a32(B v)
-{	dest_addr &= 0x00ffffff;
-	dest_addr |= (v&0xff) << 24;
+void BLITTER::handleWriteL(uaecptr addr, uint32 value) {
+	addr -= getHWoffset();
+	switch(addr) {
+		case 0x24:
+			D(bug("Blitter sets source to $%08lx at $%08lx", value, showPC()));
+			source_addr = value & 0xfffffffe;
+			source_addr_backup = source_addr;
+			break;
+		case 0x32:
+			D(bug("Blitter sets dest to $%08lx at $%08lx", value, showPC()));
+			dest_addr = value & 0xfffffffe;
+			dest_addr_backup = dest_addr;
+			break;
+		default: BASE_IO::handleWriteL(addr+getHWoffset(), value); break;
+	}
+
 }
 
-void BLITTER::STORE_B_ff8a33(B v)
-{	dest_addr &= 0xff00ffff;
-	dest_addr |= (v&0xff) << 16;
-}
-
-void BLITTER::STORE_B_ff8a34(B v)
-{	dest_addr &= 0xffff00ff;
-	dest_addr |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a35(B v)
-{	dest_addr &= 0xffffff00;
-	dest_addr |= (v & 0xfe);	// ignore LSB
-}
-
-void BLITTER::STORE_B_ff8a28(B v)
-{	end_mask_1 &= 0x00ff;
-	end_mask_1 |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a29(B v)
-{	end_mask_1 &= 0xff00;
-	end_mask_1 |= (v&0xff);
-}
-
-void BLITTER::STORE_B_ff8a2a(B v)
-{	end_mask_2 &= 0x00ff;
-	end_mask_2 |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a2b(B v)
-{	end_mask_2 &= 0xff00;
-	end_mask_2 |= (v&0xff);
-}
-
-void BLITTER::STORE_B_ff8a2c(B v)
-{	end_mask_3 &= 0x00ff;
-	end_mask_3 |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a2d(B v)
-{	end_mask_3 &= 0xff00;
-	end_mask_3 |= (v&0xff);
-}
-
-void BLITTER::STORE_B_ff8a36(B v)
-{	x_count &= 0x00ff;
-	x_count |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a37(B v)
-{	x_count &= 0xff00;
-	x_count |= (v&0xff);
-}
-
-void BLITTER::STORE_B_ff8a38(B v)
-{	y_count &= 0x00ff;
-	y_count |= (v&0xff) << 8;
-}
-
-void BLITTER::STORE_B_ff8a39(B v)
-{	y_count &= 0xff00;
-	y_count |= (v&0xff);
+B BLITTER::LOAD_B_ff8a3c(void)
+{
+	return line_num & 0x3f;
 }
 
 void BLITTER::STORE_B_ff8a3a(B v)
@@ -629,14 +534,14 @@ void BLITTER::STORE_B_ff8a3b(B v)
 }
 
 void BLITTER::STORE_B_ff8a3c(B v)
-{	
+{	// 765_3210  Line-Number
 	line_num   = (UB) v & 0x3f;
 	if ((y_count !=0) && (v & 0x80)) /* Busy bit set and lines to blit? */
 		blit = true;
 }
 
 void BLITTER::STORE_B_ff8a3d(B v)
-{	
+{	// 76__3210  Skew
 	NFSR = (v & 0x40) != 0;					
 	FXSR = (v & 0x80) != 0;					
 	skewreg = (unsigned char) v & 0xcf;	/* h/ware reg mask %11001111 !*/	
@@ -650,6 +555,24 @@ void BLITTER::Do_Blit(void)
 	SHOWPARAMS;
 #endif
 
+	if (source_addr != source_addr_backup) {
+		D(bug("Blitter starts with obsolete source addr $%08lx, should be $%08lx", source_addr, source_addr_backup));
+		source_addr = source_addr_backup;
+	}
+	if (dest_addr != dest_addr_backup) {
+		D(bug("Blitter starts with obsolete dest addr $%08lx, should be $%08lx", dest_addr, dest_addr_backup));
+		dest_addr = dest_addr_backup;
+	}
+
+	if (source_addr <= 0x800 || (source_addr >= 0x0e80000 && source_addr < 0x1000000)) {
+		panicbug("Blitter Source address out of range: $%08lx", source_addr);
+		return;
+	}
+	if (dest_addr <= 0x800 || (dest_addr >= 0x0e00000 && dest_addr < 0x1000000)) {
+		panicbug("Blitter Destination address out of range: $%08lx", dest_addr);
+		return;
+	}
+
 	if (source_x_inc < 0) {
 		do_hop_op_N[hop][op]( *this );
 	}
