@@ -46,6 +46,9 @@ void DirtyRects::resizeDirty(int width, int height)
 		delete dirtyMarker;
 	}
 
+	areaW = width;
+	areaH = height;
+
 	dirtyW = width>>4;
 	if (width & 15) {
 		dirtyW++;
@@ -58,6 +61,10 @@ void DirtyRects::resizeDirty(int width, int height)
 
 	/* Will refresh everything */
 	memset(dirtyMarker, 1, dirtyW * dirtyH);
+	minDirtX = 0;
+	minDirtY = 0;
+	maxDirtX = areaW-1;
+	maxDirtY = areaH-1;
 }
 
 Uint8 *DirtyRects::getDirtyRects(void)
@@ -67,6 +74,17 @@ Uint8 *DirtyRects::getDirtyRects(void)
 
 void DirtyRects::setDirtyRect(int x, int y, int w, int h)
 {
+	// adjust minDirt and maxDirt points
+	if (x < minDirtX)
+		minDirtX = x & 0xfffffff0;  // rounded down to multiple of 16
+	if (y < minDirtY)
+		minDirtY = y & 0xfffffff0;  // rounded down to multiple of 16
+	if (x+w-1 > maxDirtX)
+		maxDirtX = x+w-1;
+	if (y+h-1 > maxDirtY)
+		maxDirtY = y+h-1;
+
+	// mark affected area in our marker map as dirty
 	int x2 = x+w;
 	if (x2 & 15) {
 		x2 = (x2|15)+1;
@@ -78,7 +96,7 @@ void DirtyRects::setDirtyRect(int x, int y, int w, int h)
 	}
 	y2>>=4;
 	int x1 = x>>4, y1 = y>>4;
-
+    
 	for (y=y1;y<y2;y++) {
 		for(x=x1;x<x2;x++) {
 			if ((x>=0) && (x<dirtyW) && (y>=0) && (y<dirtyH)) {
@@ -88,11 +106,47 @@ void DirtyRects::setDirtyRect(int x, int y, int w, int h)
 	}
 }
 
+void DirtyRects::setDirtyLine(int x1, int y1, int x2, int y2)
+{
+	int min_x,min_y, max_x, max_y;
+	if (x1<=x2) {
+		min_x=x1;
+		max_x=x2;
+	}
+	else {
+		min_x=x2;
+		max_x=x1;
+	}
+	if (y1<=y2) {
+		min_y=y1;
+		max_y=y2;
+	}
+	else {
+		min_y=y2;
+		max_y=y1;
+	}
+	setDirtyRect(min_x,min_y,max_x-min_x+1,max_y-min_y+1);
+}
+
+
 void DirtyRects::clearDirtyRects(void)
 {
+	// clear marker map
 	memset(dirtyMarker, 0, dirtyW * dirtyH);
+
+	// reset dirt points
+	minDirtX = areaW;
+	minDirtY = areaH;
+	maxDirtX = 0;
+	maxDirtY = 0;
 }
 
+bool DirtyRects::hasDirtyRect(void)
+{
+	return minDirtX<=maxDirtX;
+}
+
+
 int DirtyRects::getDirtyWidth(void)
 {
 	return dirtyW;
@@ -102,3 +156,23 @@ int DirtyRects::getDirtyHeight(void)
 {
 	return dirtyH;
 }
+
+int DirtyRects::getMinDirtX(void)
+{
+	return minDirtX;
+}
+
+int DirtyRects::getMinDirtY(void)
+{
+	return minDirtY;
+}
+
+int DirtyRects::getMaxDirtX(void)
+{
+	return maxDirtX;
+}
+
+int DirtyRects::getMaxDirtY(void)
+{
+	return maxDirtY;
+}
