@@ -24,7 +24,9 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 #include "sdlgui.h"
 #include "file.h"
 #include "ata.h"
