@@ -1,3 +1,27 @@
+/*
+ * m68k.h - machine dependent bits
+ *
+ * Copyright (c) 2001-2004 Milan Jurik of ARAnyM dev team (see AUTHORS)
+ * 
+ * Inspired by Christian Bauer's Basilisk II
+ *
+ * This file is part of the ARAnyM project which builds a new and powerful
+ * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
+ *
+ * ARAnyM is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * ARAnyM is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with ARAnyM; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ */
  /* 
   * UAE - The Un*x Amiga Emulator
   * 
@@ -11,22 +35,27 @@
 
 #ifdef OPTIMIZED_FLAGS
 
-#if defined(__i386__) && defined(X86_ASSEMBLY)
+#if (defined(CPU_i386) && defined(X86_ASSEMBLY)) || (defined(CPU_x86_64) && defined(X86_64_ASSEMBLY))
+
+# include <cstdlib>
 
 #ifndef SAHF_SETO_PROFITABLE
 
+/* PUSH/POP instructions are naturally 64-bit sized on x86-64, thus
+   unsigned long hereunder is either 64-bit or 32-bit wide depending
+   on the target.  */
 struct flag_struct {
-    uae_u32 cznv;
-    uae_u32 x;
+    unsigned long cznv;
+    unsigned long x;
 };
 
 #define FLAGVAL_Z	0x40
 #define FLAGVAL_N	0x80
 
-#define SET_ZFLG(y)	(regflags.cznv = (regflags.cznv & ~0x40) | (((y) & 1) << 6))
-#define SET_CFLG(y)	(regflags.cznv = (regflags.cznv & ~1) | ((y) & 1))
-#define SET_VFLG(y)	(regflags.cznv = (regflags.cznv & ~0x800) | (((y) & 1) << 11))
-#define SET_NFLG(y)	(regflags.cznv = (regflags.cznv & ~0x80) | (((y) & 1) << 7))
+#define SET_ZFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x40) | (((y) & 1) << 6))
+#define SET_CFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~1) | ((y) & 1))
+#define SET_VFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x800) | (((y) & 1) << 11))
+#define SET_NFLG(y)	(regflags.cznv = (((uae_u32)regflags.cznv) & ~0x80) | (((y) & 1) << 7))
 #define SET_XFLG(y)	(regflags.x = (y))
 
 #define GET_ZFLG	((regflags.cznv >> 6) & 1)
@@ -44,7 +73,7 @@ struct flag_struct {
 
 extern struct flag_struct regflags __asm__ ("regflags");
 
-static __inline__ int cctrue(int cc)
+static inline int cctrue(int cc)
 {
     uae_u32 cznv = regflags.cznv;
     switch(cc){
@@ -74,86 +103,86 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_testl(v) \
   __asm__ __volatile__ ("andl %1,%1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "r" (v) : "cc")
 
 #define optflag_testw(v) \
   __asm__ __volatile__ ("andw %w1,%w1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "r" (v) : "cc")
 
 #define optflag_testb(v) \
   __asm__ __volatile__ ("andb %b1,%b1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "q" (v) : "cc")
 
 #define optflag_addl(v, s, d) do { \
   __asm__ __volatile__ ("addl %k2,%k1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_addw(v, s, d) do { \
   __asm__ __volatile__ ("addw %w2,%w1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_addb(v, s, d) do { \
   __asm__ __volatile__ ("addb %b2,%b1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=q" (v) : "qmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_subl(v, s, d) do { \
   __asm__ __volatile__ ("subl %k2,%k1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_subw(v, s, d) do { \
   __asm__ __volatile__ ("subw %w2,%w1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=r" (v) : "rmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_subb(v, s, d) do { \
   __asm__ __volatile__ ("subb %b2,%b1\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv), "=q" (v) : "qmi" (s), "1" (d) : "cc"); \
     COPY_CARRY; \
     } while (0)
 
 #define optflag_cmpl(s, d) \
   __asm__ __volatile__ ("cmpl %k1,%k2\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "rmi" (s), "r" (d) : "cc")
 
 #define optflag_cmpw(s, d) \
   __asm__ __volatile__ ("cmpw %w1,%w2\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "rmi" (s), "r" (d) : "cc")
 
 #define optflag_cmpb(s, d) \
   __asm__ __volatile__ ("cmpb %b1,%b2\n\t" \
-			"pushfl\n\t" \
-			"popl %0\n\t" \
+			"pushf\n\t" \
+			"pop %0\n\t" \
 			: "=r" (regflags.cznv) : "qmi" (s), "q" (d) : "cc")
 
 #else
@@ -187,7 +216,7 @@ struct flag_struct {
 
 extern struct flag_struct regflags __asm__ ("regflags");
 
-static __inline__ int cctrue(int cc)
+static inline int cctrue(int cc)
 {
     uae_u32 cznv = regflags.cznv;
     switch(cc){
@@ -216,11 +245,18 @@ static __inline__ int cctrue(int cc)
     return 0;
 }
 
+/* Manually emit LAHF instruction so that 64-bit assemblers can grok it */
+#if defined CPU_x86_64 && defined __GNUC__
+#define ASM_LAHF ".byte 0x9f"
+#else
+#define ASM_LAHF "lahf"
+#endif
+
 /* Is there any way to do this without declaring *all* memory clobbered?
    I.e. any way to tell gcc that some byte-sized value is in %al? */
 #define optflag_testl(v) \
   __asm__ __volatile__ ("andl %0,%0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -228,7 +264,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_testw(v) \
   __asm__ __volatile__ ("andw %w0,%w0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -236,7 +272,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_testb(v) \
   __asm__ __volatile__ ("andb %b0,%b0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -244,7 +280,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_addl(v, s, d) do { \
   __asm__ __volatile__ ("addl %k1,%k0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -254,7 +290,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_addw(v, s, d) do { \
   __asm__ __volatile__ ("addw %w1,%w0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -264,7 +300,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_addb(v, s, d) do { \
   __asm__ __volatile__ ("addb %b1,%b0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -274,7 +310,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_subl(v, s, d) do { \
   __asm__ __volatile__ ("subl %k1,%k0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -284,7 +320,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_subw(v, s, d) do { \
   __asm__ __volatile__ ("subw %w1,%w0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -294,7 +330,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_subb(v, s, d) do { \
    __asm__ __volatile__ ("subb %b1,%b0\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -304,7 +340,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_cmpl(s, d) \
   __asm__ __volatile__ ("cmpl %k0,%k1\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -312,7 +348,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_cmpw(s, d) \
   __asm__ __volatile__ ("cmpw %w0,%w1\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -320,7 +356,7 @@ static __inline__ int cctrue(int cc)
 
 #define optflag_cmpb(s, d) \
   __asm__ __volatile__ ("cmpb %b0,%b1\n\t" \
-			"lahf\n\t" \
+			ASM_LAHF "\n\t" \
 			"seto %%al\n\t" \
 			"movb %%al,regflags\n\t" \
 			"movb %%ah,regflags+1\n\t" \
@@ -328,7 +364,7 @@ static __inline__ int cctrue(int cc)
 
 #endif
 
-#elif defined(__sparc__) && (defined(SPARC_V8_ASSEMBLY) || defined(SPARC_V9_ASSEMBLY))
+#elif defined(CPU_sparc) && (defined(SPARC_V8_ASSEMBLY) || defined(SPARC_V9_ASSEMBLY))
 
 struct flag_struct {
     unsigned char nzvc;
@@ -359,7 +395,7 @@ extern struct flag_struct regflags;
 
 #define COPY_CARRY (regflags.x = regflags.nzvc)
 
-static __inline__ int cctrue(int cc)
+static inline int cctrue(int cc)
 {
     uae_u32 nzvc = regflags.nzvc;
     switch(cc){
@@ -1021,7 +1057,7 @@ extern struct flag_struct regflags;
 
 #define COPY_CARRY (SET_XFLG (GET_CFLG))
 
-static __inline__ int cctrue(const int cc)
+static inline int cctrue(const int cc)
 {
     switch(cc){
      case 0: return 1;                       /* T */
