@@ -1,8 +1,9 @@
 /*
  * newcpu.cpp - CPU emulation
  *
- * Copyright (c) 2001-2004 Milan Jurik of ARAnyM dev team (see AUTHORS)
+ * Copyright (c) 2010 ARAnyM dev team (see AUTHORS)
  * 
+ *
  * Inspired by Christian Bauer's Basilisk II
  *
  * This file is part of the ARAnyM project which builds a new and powerful
@@ -66,9 +67,6 @@ struct flag_struct regflags;
 #ifdef EXCEPTIONS_VIA_LONGJMP
 JMP_BUF excep_env;
 #endif
-#ifdef DISDIP
-JMP_BUF loop_env;
-#endif
 /* Opcode of faulting instruction */
 uae_u16 last_op_for_exception_3;
 /* PC at fault time */
@@ -76,14 +74,6 @@ uaecptr last_addr_for_exception_3;
 /* Address that generated the exception */
 uaecptr last_fault_for_exception_3;
 
-#ifdef DISDIP
-/* Is opcode label table initialized?*/
-bool initial;
-
-/* Jump table */
-void *op_smalltbl_0_lab[65536];
-#endif
-
 int areg_byteinc[] = { 1,1,1,1,1,1,1,2 };
 int imm8_table[] = { 8,1,2,3,4,5,6,7 };
 
@@ -251,15 +241,6 @@ static void build_cpufunctbl (void)
 	if (tbl[i].specific)
 	    cpufunctbl[cft_map (tbl[i].opcode)] = tbl[i].handler;
     }
-#ifdef DISDIP
-    for (opcode = 0; opcode < 65536; opcode++)
-	(*cpufunctbl[opcode])(opcode);
-#if 0
-    for (opcode = 0; opcode < 65536; opcode++)
-	panicbug("%lx 0x%08x", opcode, op_smalltbl_0_lab[opcode]);
-#endif
-    initial = true;
-#endif
 }
 
 void init_m68k (void)
@@ -962,6 +943,16 @@ static void Interrupt(int nr)
     // regs.spcflags |= SPCFLAG_INT; (disabled by joy)
 }
 
+static void SCCInterrupt(int nr)
+{
+    // fprintf(stderr, "CPU: in SCCInterrupt\n");
+    lastint_regs = regs;
+    lastint_no = 5;// ex 5
+    Exception(nr, 0);
+
+    regs.intmask = 5;// ex 5
+}
+
 static void MFPInterrupt(int nr)
 {
     // fprintf(stderr, "CPU: in MFPInterrupt\n");
@@ -1343,7 +1334,7 @@ void m68k_emulop(uae_u32 opcode)
 {
 	struct M68kRegisters r;
 	int i;
-	
+
 	for (i=0; i<8; i++) {
 		r.d[i] = m68k_dreg(regs, i);
 		r.a[i] = m68k_areg(regs, i);
@@ -1408,22 +1399,7 @@ void m68k_natfeat_call(void)
 	MakeFromSR();
 }
 
-#ifdef DISDIP
-void REGPARAM2 op_illg (uae_u32 opc)
-{
-	uae_u32 xpc;
-	if (initial) {
-		opcode = opc;
-		goto op_illg_lab;
-	}
-	opc = cft_map(opc);
-	op_smalltbl_0_lab[opc] = &&op_illg_lab;
-	return;
-op_illg_lab:
-	opcode = cft_map(opcode);
-#else
 void REGPARAM2 op_illg (uae_u32 opcode)
-#endif
 {
 #if DEBUG
 	uaecptr pc = m68k_getpc ();
@@ -1431,20 +1407,12 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 
 	if ((opcode & 0xF000) == 0xA000) {
 	Exception(0xA,0);
-#ifdef DISDIP
-	LONGJMP(loop_env, 0);
-#else
 	return;
-#endif
 	}
 
 	if ((opcode & 0xF000) == 0xF000) {
 	Exception(0xB,0);
-#ifdef DISDIP
-	LONGJMP(loop_env, 0);
-#else
 	return;
-#endif
 	}
 
 	D(bug("Illegal instruction: %04x at %08lx", opcode, pc));
@@ -1453,12 +1421,7 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 #endif
 
 	Exception (4,0);
-#ifdef DISDIP
-}
-	LONGJMP(loop_env, 0);
-#else
 	return;
-#endif
 }
 
 static uaecptr last_trace_ad = 0;
@@ -1500,7 +1463,7 @@ static void do_trace (void)
 
 #define SERVE_VBL_MFP(resetStop)							\
 {															\
-	if (SPCFLAGS_TEST( SPCFLAG_INT3|SPCFLAG_VBL|SPCFLAG_INT5|SPCFLAG_MFP )) {		\
+	if (SPCFLAGS_TEST( SPCFLAG_INT3|SPCFLAG_VBL|SPCFLAG_INT5|SPCFLAG_SCC|SPCFLAG_MFP )) {		\
 		if (SPCFLAGS_TEST( SPCFLAG_INT3 )) {					\
 			if (3 > regs.intmask) {							\
 				Interrupt(3);								\
@@ -1528,6 +1491,20 @@ static void do_trace (void)
 					SPCFLAGS_CLEAR( SPCFLAG_STOP );			\
 			}												\
 		}													\
+		if (SPCFLAGS_TEST( SPCFLAG_SCC )) {					\
+			if (5 > regs.intmask) {						\
+				int vector_number=SCCdoInterrupt();			\
+				if(vector_number){					\
+					 SCCInterrupt(vector_number);	        	\
+					regs.stopped = 0;				\
+					SPCFLAGS_CLEAR( SPCFLAG_SCC);		\
+					if (resetStop)					\
+						SPCFLAGS_CLEAR( SPCFLAG_STOP );		\
+				}							\
+				else							\
+					SPCFLAGS_CLEAR( SPCFLAG_SCC );		\
+			}								\
+		}									\
 		if (SPCFLAGS_TEST( SPCFLAG_MFP )) {					\
 			if (6 > regs.intmask) {							\
 				int vector_number = MFPdoInterrupt();		\
@@ -1558,7 +1535,7 @@ int m68k_do_specialties(void)
 #ifdef USE_JIT
 	// Block was compiled
 	SPCFLAGS_CLEAR( SPCFLAG_JIT_END_COMPILE );
-	
+
 	// Retain the request to get out of compiled code until
 	// we reached the toplevel execution, i.e. the one that
 	// can compile then run compiled code. This also means
@@ -1584,7 +1561,7 @@ int m68k_do_specialties(void)
 
 	SERVE_VBL_MFP(false);
 
-/*  
+/*
 // do not understand the INT vs DOINT stuff so I disabled it (joy)
 	if (regs.spcflags & SPCFLAG_INT) {
 		regs.spcflags &= ~SPCFLAG_INT;
@@ -1603,9 +1580,6 @@ void m68k_do_execute (void)
 {
     uae_u32 pc;
     uae_u32 opcode;
-#ifdef DISDIP
-    if (SETJMP(loop_env)) return;
-#endif	    
     for (;;) {
 	regs.fault_pc = pc = m68k_getpc();
 #ifdef FULL_HISTORY
@@ -1640,12 +1614,10 @@ void m68k_do_execute (void)
 	cpu_check_ticks();
 	regs.fault_pc = m68k_getpc();
 
-#ifndef DISDIP
 	if (SPCFLAGS_TEST(SPCFLAG_ALL_BUT_EXEC_RETURN)) {
-	    if (m68k_do_specialties())
-		return;
+		if (m68k_do_specialties())
+			return;
 	}
-#endif
     }
 }
 
